from pathlib import Path
from configparser import ConfigParser
import urllib.request

from atuin.core import CORE_LOGGER as log
import atuin.defaults as default


def check_atuin_init(dir=None):
    """Check if atuin has been initialized in the current directory
        by checking if the '.atuin' file exists"""
    if dir is not None:
        return (dir / default.ATUIN_FILE).exists()

    return default.ATUIN_FILE.exists()


def init_atuin_directory(directory, config_location, force=False):
    """Initializes the directory given to contain a .atuin file
        force will refresh config and try to refresh templates"""

    if directory is None:
        directory = default.ATUIN_FILE
    else:
        directory = Path(directory) / default.ATUIN_FILE

    if '..' in directory.parts:
        # TODO: Better Error Message
        log.warning('Cannot initialize directories above the \
                current working directory: {}'.format(directory.parent))
        return False

    # try to make atuin directory
    try:
        directory.mkdir(parents=True, exist_ok=True)
    except Exception as e:
        log.error(e)
        return False

    config = directory / default.CONFIG_FILE
    # reload config from given config or default config if non-existant or forced
    if config.exists() and not force:
        log.info('Atuin configuration already exists in {}'.format(directory))
    else:
        config_string = resolve_config(config_location)
        try:
            cf = config.open(mode='w')
            cf.write(config_string)
            cf.close()
        except Exception as e:
            log.error(e)
            return False

    # Load Config and initialize Template Values
    try:
        curr_config = ConfigParser()
        curr_config.read([default.INTERNAL_PATH / default.CONFIG_FILE, config])

        for key in curr_config['templates']:
            init_templates(key, curr_config.get('templates', key), directory, force)
    except Exception as e:
        log.error(e)
        log.warning('Unable to initialize all template values')
        return False

    return True


def resolve_config(config):
    """Returns the config for atuin directory given a default, or url"""
    if config is not None:
        log.debug('Initializing Atuin configuration from {}'.format(config))

        try:
            resp = urllib.request.urlopen(config)
            config = resp.read().decode()

            if verify_config(config):
                return config
        except Exception as e:
            log.error(e)

    log.debug('Using default atuin configuration')
    config = (default.INTERNAL_PATH / default.CONFIG_FILE).open().read()
    return config


def verify_config(config_string):
    """Verify the config has at least the minimum required value:
            config['templates']['default']"""
    try:
        test = ConfigParser()
        test.read_string(config_string)
        if 'templates' in test.sections() and test.has_option('templates', 'default'):
            log.debug(test.get('templates', 'default'))
            return True
    except Exception as e:
        log.warning(e)
    return False


def init_templates(key, template, dir=default.ATUIN_FILE, force=False):
    """Load templates, looking checking existence and trying to resolve
        non-existing templates"""

    template_dir = dir / default.TEMPLATE_PATH

    if not template_dir.exists():
        try:
            template_dir.mkdir()
        except Exception as e:
            log.warning(e)

    template_file = template_dir / template

    if not template_file.exists() or force:
        log.debug('Initializing template <{}> from {}'.format(key, template))
        template_string = resolve_template(template)
        try:
            template_file.open(mode='w').write(template_string)
        except (Exception):
            pass


def resolve_template(template_name, dir=default.INTERNAL_PATH / default.TEMPLATE_PATH, uri=None):
    """Produce a string given a template name"""
    template = dir / template_name

    if not template.is_file():
        log.warning('Unable to find {} at {}'.format(template_name, template))
        return resolve_template('default.template')
    else:
        return template.open().read()



import atuin
from atuin.commands import subcommand, argument
from atuin.core import CORE_LOGGER as log
from atuin.core.initialize import init_atuin_directory, check_atuin_init
from atuin.core.management import get_template, create_challenge_tree
from atuin.core.management import sanitize_challenge_name, challenge_add_template


@subcommand()
def status(arg):
    """print status of current version"""
    # TODO: Actually do something useful here
    print(atuin.__version__)


@subcommand([argument('name', type=str, nargs='+', help='Challenge Name'),
             argument('-p', '--points', type=int, help='Challenge Point Value'),
             argument('-t', '--template', type=str, default='default', help='Custom Template File')],
            'Create a challenge or other object')
def create(arg):
    """Creates a challenge with given templates"""
    if not check_atuin_init():
        log.warning('Current directory is not initialized')
        exit(1)

    template = get_template(arg.template)

    for name in arg.name:
        filename = sanitize_challenge_name(name)
        path = create_challenge_tree(filename)

        if path is None:
            exit(1)

        challenge_add_template(name, filename, path, template, arg.points)


@subcommand([argument('-f', '--force', action='store_true', help='Force reinitialize'),
             argument('dir', type=str, nargs='?', help='Directory to initialize'),
             argument('--config', type=str, help='File or URI for configuration options')],
            'Initialize Atuin')
def init(arg):
    """Initializes the current or given directory for atuin"""
    if not init_atuin_directory(arg.dir, arg.config, arg.force):
        exit(1)
    else:
        exit(0)



from logging import StreamHandler, DEBUG, getLogger, Formatter
from colorama import Fore, Back, Style, init


def init_logging(level):
    # colorama initialization for logging
    init()
    getLogger().setLevel(level)


class ColourStreamHandler(StreamHandler):
    """ A colorized output SteamHandler """

    # Some basic colour scheme defaults
    colours = {
        'DEBUG': Fore.CYAN + Style.BRIGHT,
        'INFO': Fore.WHITE + Style.BRIGHT,
        'WARNING': Fore.YELLOW + Style.BRIGHT,
        'ERROR': Fore.RED + Style.BRIGHT,
        'CRITICAL': Back.RED + Fore.WHITE + Style.BRIGHT
    }

    def emit(self, record):
        try:
            message = self.format(record)
            output = self.colours[record.levelname] + message + Style.RESET_ALL
            self.stream.write(output)
            self.stream.write(getattr(self, 'terminator', '\n'))
            self.flush()
        except (KeyboardInterrupt, SystemExit):
            raise
        except (Exception):
            self.handleError(record)


def getColorLogger(name=None, format='%(message)s', level=DEBUG):
    """ Get and initialize a colourised logging instance if the system supports
    it as defined by the log.has_colour
    """
    log = getLogger(name)
    handler = ColourStreamHandler()
    # sets what level this handler can manage
    # root logging level controls output level
    handler.setLevel(level)
    handler.setFormatter(Formatter(format))
    log.addHandler(handler)
    log.propagate = True
    return log


LOG_FORMAT_FULL = '%(name)s [%(levelname)s]: %(msg)s'
LOG_FORMAT_NAME = '%(name)s: %(msg)s'
LOG_FORMAT_LEVEL = '[%(levelname)s]: %(msg)s'
LOG_FORMAT_MSG = '%(msg)s'


def testLogger(logger):
    logger.debug('Test DEBUG')
    logger.info('Test INFO')
    logger.warning('Test WARNING')
    logger.error('Test ERROR')
    logger.critical('Test CRITICAL')



import logging
from atuin.commands import parser
from atuin.commands import *

__LOG_LEVEL = {0: logging.WARNING, 1: logging.INFO, 2: logging.DEBUG}


def main():

    args = parser.parse_args()
    if args.subcommand is None:
        parser.print_help()
    else:
        logging.getLogger().setLevel(__LOG_LEVEL[min(2, args.verbose)])
        args.func(args)


if __name__ == '__main__':
    main()



from configparser import ConfigParser
from collections import defaultdict
from pathlib import Path
import string
import atuin.defaults as default
from atuin.core import CORE_LOGGER as log


def get_template(template_name):
    """Gets a template based off name or returns default.template if not found"""
    config = ConfigParser()
    config.read(default.ATUIN_FILE / default.CONFIG_FILE)
    template_name = config.get('templates', template_name, fallback=template_name)
    log.debug('Trying getting template {}'.format(template_name))

    template = default.ATUIN_FILE / default.TEMPLATE_PATH / template_name

    # TODO:make this search more robust and potentially return something else
    if template.is_file():
        return template
    else:
        return default.INTERNAL_PATH / 'default.template'


def sanitize_challenge_name(challenge_name):
    """Produces a set filename for any challenge name by removing 'invalid'
        characters, eliminating spaces, and making it lowercase"""
    valid_chars = '-_. {}{}'.format(string.ascii_letters, string.digits)
    filename = ''.join(c for c in challenge_name if c in valid_chars)
    filename = filename.title()
    filename = filename.replace(' ', '')
    return filename


def create_challenge_tree(challenge_name):
    """Creates folder for templates if they don't already exist
        Returns the given path,"""

    path = Path(challenge_name)

    if path.is_dir():
        return path
    else:
        try:
            path.mkdir(parents=True, exist_ok=True)
            return path
        except Exception as e:
            log.error('Failed to create challenge directory')
            log.debug(e)
        # TODO: Make this more robust
    return None


def challenge_add_template(name, filename, path, template, points):
    """Adds a given template to a challenge"""
    log.info('Creating {} in {} with from {}'.format(filename, path, template))

    template_string = template.open().read()
    template_string = template_string.format_map(defaultdict(str, title=name, points=points))
    suffix = Path(template.stem).suffix
    stem = '{}{}'.format(filename, suffix)
    log.debug("{} - {}".format(path, stem))
    file = path / stem

    if file.exists():
        log.warning('Challenge Already Exists')
        return

    file.write_text(template_string)



from atuin.core.colorlogging import init_logging
from atuin.core.colorlogging import getColorLogger
# from atuin.core.colorlogging import testLogger
from atuin.core.colorlogging import LOG_FORMAT_MSG
import logging


init_logging(logging.DEBUG)
CORE_LOGGER = getColorLogger(__name__, LOG_FORMAT_MSG)
# colorlogging.testLogger(CORE_LOGGER)



import argparse
import atuin

parser = argparse.ArgumentParser(description=None)
parser.add_argument('--version', action='version', version=atuin.__version__)
parser.add_argument('-v', '--verbose', action='count', default=0)
subparsers = parser.add_subparsers(metavar='command', dest="subcommand")


def subcommand(args=[], help=None, name=None, parent=subparsers):
    """Decorator for wrapping argparse capabilities"""
    def decorator(func):
        nonlocal name
        if name is None:
            name = func.__name__.replace('_', '-')
        parser = parent.add_parser(name, help=help, description=func.__doc__)

        for arg in args:
            parser.add_argument(*arg[0], **arg[1])
        parser.set_defaults(func=func)

    return decorator


def argument(*name_or_flags, **kwargs):
    """Helper function to satisfy argparse.ArgumentParser.add_argument()'s
    input argument syntax"""
    return (list(name_or_flags), kwargs)


__all__ = ['base']



from setuptools import setup, find_packages
import atuin

setup(
    name='atuin',
    version=atuin.__version__,
    description='''A tool for challenge creation and management
                    for CSTEM/Roboplay Competition''',
    author='Steven Herman',
    author_email='stejher@gmail.com',
    license='GPL3',
    packages=find_packages(),
    install_requires=[
        'colorama',
    ],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'atuin = atuin.__main__:main'
        ]
    })
